@@ -12,7 +12,7 @@ import typecheck
 
 def check_type(typ, obj):
 	typecheck.check_type(typ, None, obj)
-
+	
 class Test_Or(TestCase):
 	def setUp(self):
 		from typecheck import Or
@@ -22,6 +22,11 @@ class Test_Or(TestCase):
 	
 		self.or_type = or_type
 		
+	def test_IsOneOf_alias(self):
+		from typecheck import Or, IsOneOf
+		
+		assert Or is IsOneOf
+		
 	def test_constructor(self):
 		from typecheck import Or
 		
@@ -113,6 +118,11 @@ class Test_Or(TestCase):
 		test_hash(eq_tests, ne_tests)
 			
 class Test_And(TestCase):
+	def test_IsAllOf_alias(self):
+		from typecheck import And, IsAllOf
+		
+		assert And is IsAllOf
+
 	def test_success(self):
 		from typecheck import And
 	
@@ -184,6 +194,11 @@ class Test_And(TestCase):
 		test_hash(eq_tests, ne_tests)
 
 class Test_Not(TestCase):
+	def test_IsNoneOf_alias(self):
+		from typecheck import Not, IsNoneOf
+		
+		assert Not is IsNoneOf
+
 	def test_constructor(self):
 		from typecheck import Not
 		
@@ -541,13 +556,11 @@ class Test_IsCallable(TestCase):
 		try:
 			foo(5)
 		except TypeCheckError, e:
-			assert isinstance(e.internal, _TC_IndexError)
-			assert e.internal.index == 0
-			assert isinstance(e.internal.inner, _TC_TypeError)
-			assert e.internal.inner.right == 'a callable'
-			assert e.internal.inner.wrong == int
+			assert isinstance(e.internal, _TC_TypeError)
+			assert e.internal.right == 'a callable'
+			assert e.internal.wrong == int
 			
-			self.assertEquals(str(e), "in (5,), at index 0, expected a callable, got <type 'int'>")
+			self.assertEquals(str(e), "Argument a: for 5, expected a callable, got <type 'int'>")
 		else:
 			raise AssertionError("Failed to raise TypeCheckError")
 			
@@ -865,7 +878,7 @@ class Test_YieldSeq(TestCase):
 			assert isinstance(e.internal.inner, _TC_TypeError)
 			assert e.internal.inner.right == int
 			assert e.internal.inner.wrong == float
-			self.assertEqual(str(e), "At yield #2: in 7.0, expected <type 'int'>, got <type 'float'>")
+			self.assertEqual(str(e), "At yield #2: for 7.0, expected <type 'int'>, got <type 'float'>")
 		else:
 			raise AssertionError("Did not raise TypeCheckError")
 			
@@ -930,6 +943,11 @@ class Test_YieldSeq(TestCase):
 		test_hash(eq_tests, ne_tests)
 			
 class Test_Xor(TestCase):
+	def test_IsOnlyOneOf_alias(self):
+		from typecheck import Xor, IsOnlyOneOf
+		
+		assert Xor is IsOnlyOneOf
+
 	def test_no_parameters(self):
 		from typecheck import Xor
 		
@@ -1025,7 +1043,295 @@ class Test_Xor(TestCase):
 			(Xor(int, float), Or(int, float)) ]
 			
 		test_hash(eq_tests, ne_tests)
+		
+class Test_Exact(TestCase):
+	def test_constructor(self):
+		from typecheck import Exact
+		
+		try:
+			Exact()
+		except TypeError, e:
+			assert str(e) == "__init__() takes exactly 2 arguments (1 given)"
+		else:
+			raise AssertionError("Failed to raise TypeError")
+			
+		try:
+			Exact(4, 5)
+		except TypeError, e:
+			assert str(e) == "__init__() takes exactly 2 arguments (3 given)"
+		else:
+			raise AssertionError("Failed to raise TypeError")
+			
+	def test_equality(self):
+		from typecheck import Exact
+		
+		eq_tests = [
+					(Exact(4), Exact(4)),
+					(Exact([4, 5]), Exact([4, 5])),
+					(Exact(Exact), Exact(Exact)) ]
+					
+		ne_tests = [
+					(Exact(4), Exact(5)),
+					(Exact([5, 4]), Exact([4, 5])),
+					(Exact(Exact), Exact(object)),
+					(Exact(object), Exact(dict)) ]
+					
+		test_equality(eq_tests, ne_tests)
+		
+	def test_hash(self):
+		from typecheck import Exact
+		
+		eq_tests = [
+					(Exact(4), Exact(4)),
+					(Exact([4, 5]), Exact([4, 5])),
+					(Exact(Exact), Exact(Exact)) ]
+					
+		ne_tests = [
+					(Exact(4), Exact(5)),
+					(Exact([5, 4]), Exact([4, 5])),
+					(Exact(Exact), Exact(object)),
+					(Exact(object), Exact(dict)),
+					(Exact([5, 4]), Exact(set([]))) ]
+					
+		test_hash(eq_tests, ne_tests)
+		
+	def test_pass(self):
+		from typecheck import Exact, Or
+		
+		for obj in (5, [5, 6], Exact, Or(int, float)):
+			check_type(Exact(obj), obj)
+			
+	def test_fail_1(self):
+		from typecheck import Exact, _TC_ExactError
+		
+		try:
+			check_type(Exact(5), 6)
+		except _TC_ExactError, e:
+			assert e.wrong == 6
+			assert e.right == 5
+		else:
+			raise AssertionError("Failed to raise _TC_ExactError")
+			
+	def test_fail_2(self):
+		from typecheck import Exact, _TC_ExactError
+		
+		try:
+			check_type(Exact(Exact), Exact(5))
+		except _TC_ExactError, e:
+			assert e.right == Exact
+			assert e.wrong == Exact(5)
+		else:
+			raise AssertionError("Failed to raise _TC_ExactError")
 
+	def test_fail_3(self):
+		from typecheck import Exact, CheckType, _TC_ExactError
+		
+		try:
+			check_type(Exact(CheckType), Exact)
+		except _TC_ExactError, e:
+			assert e.right == CheckType
+			assert e.wrong == Exact
+		else:
+			raise AssertionError("Failed to raise _TC_ExactError")
+			
+	def test_combined(self):
+		from typecheck import Exact, Or
+		
+		or_type = Or(Exact(5), Exact(6), Exact(7))
+		
+		for num in (5, 6, 7):
+			check_type(or_type, num)
+			
+class Test_Length(TestCase):
+	def test_constructor(self):
+		from typecheck import Length
+		
+		try:
+			Length()
+		except TypeError, e:
+			assert str(e) == "__init__() takes exactly 2 arguments (1 given)"
+		else:
+			raise AssertionError("Failed to raise TypeError")
+			
+		try:
+			Length(4, 5)
+		except TypeError, e:
+			assert str(e) == "__init__() takes exactly 2 arguments (3 given)"
+		else:
+			raise AssertionError("Failed to raise TypeError")
+			
+	def test_bad_argument(self):
+		from typecheck import Length
+		
+		try:
+			Length('abc')
+		except:
+			pass
+		else:
+			raise AssertionError("Failed to raise an exception")
+			
+	def test_equality(self):
+		from typecheck import Length
+		
+		eq_tests = [(Length(4), Length(4)), (Length(4.0), Length(4))]
+		ne_tests = [(Length(5), Length(4))]
+					
+		test_equality(eq_tests, ne_tests)
+		
+	def test_hash(self):
+		from typecheck import Length
+		
+		eq_tests = [(Length(4), Length(4)), (Length(4.0), Length(4))]
+		ne_tests = [(Length(5), Length(4))]
+					
+		test_hash(eq_tests, ne_tests)
+		
+	def test_pass_builtins(self):
+		from typecheck import Length
+		
+		for obj in ([4, 5], (6, 7), {5: 6, 7: 8}):
+			check_type(Length(2), obj)
+			
+	def test_pass_userdef(self):
+		from typecheck import Length
+		
+		class A(object):
+			def __len__(self):
+				return 5
+				
+		check_type(Length(5), A())
+			
+	def test_fail_1(self):
+		from typecheck import Length, _TC_LengthError
+		
+		try:
+			check_type(Length(5), [6])
+		except _TC_LengthError, e:
+			assert e.wrong == 1
+			assert e.right == 5
+		else:
+			raise AssertionError("Failed to raise _TC_LengthError")
+			
+	def test_fail_2(self):
+		from typecheck import Length, _TC_TypeError
+		
+		try:
+			check_type(Length(5), 5)
+		except _TC_TypeError, e:
+			assert e.right == "something with a __len__ method"
+			assert e.wrong == int
+		else:
+			raise AssertionError("Failed to raise _TC_TypeError")
+
+	def test_combined(self):
+		from typecheck import Length, And
+		
+		and_type = And(Length(3), tuple)
+		check_type(and_type, (5, 6, 7))
+		
+class Test_Class(TestCase):
+	def test_equality(self):
+		from typecheck import Class
+		
+		assert Class("ClassB") == Class("ClassB")
+		assert not Class("ClassB") != Class("ClassB")
+		
+		assert Class("ClassA") != Class("ClassB")
+		assert not Class("ClassA") == Class("ClassB")
+		
+		classb_1 = Class("ClassB")
+		
+		class ClassB(object):
+			pass
+			
+		classb_2 = Class("ClassB")
+		
+		assert classb_1 == classb_2
+		assert not classb_1 != classb_2
+		
+		check_type(classb_2, ClassB())
+		
+		assert classb_1 == classb_2
+		assert not classb_1 != classb_2
+		
+	def test_hash(self):
+		from typecheck import Class
+		
+		assert hash(Class("ClassB")) == hash(Class("ClassB"))
+		assert hash(Class("ClassA")) != hash(Class("ClassB"))
+		assert hash(Class("ClassA")) != hash("ClassA")
+		
+		classb_1 = Class("ClassB")
+		
+		class ClassB(object):
+			pass
+			
+		classb_2 = Class("ClassB")
+		
+		assert hash(classb_1) == hash(classb_2)
+		
+		check_type(classb_2, ClassB())
+		
+		assert hash(classb_1) == hash(classb_2)
+
+	def test_no_class(self):
+		from typecheck import Class, typecheck_args
+		
+		ClassB = Class("ClassB")
+		
+		@typecheck_args(ClassB)
+		def foo(a):
+			return a
+			
+		try:
+			foo(5)
+		except NameError, e:
+			self.assertEqual(str(e), "name 'ClassB' is not defined")
+		else:
+			raise AssertionError("Failed to raise NameError")
+			
+	def test_success(self):
+		from typecheck import Class, typecheck_args, Self
+		
+		ClassB = Class("ClassB")
+		
+		class ClassA(object):
+			@typecheck_args(Self(), ClassB)
+			def foo(self, a):
+				return a
+				
+		class ClassB(object):
+			@typecheck_args(Self(), ClassA)
+			def foo(self, a):	
+				return a
+				
+		ClassA().foo(ClassB())
+		ClassB().foo(ClassA())
+		
+	def test_failure(self):
+		from typecheck import Class, typecheck_args, Self
+		from typecheck import TypeCheckError, _TC_TypeError
+		
+		ClassB = Class("ClassB")
+		
+		class ClassA(object):
+			@typecheck_args(Self(), ClassB)
+			def foo(self, a):
+				return a
+				
+		class ClassB(object):
+			@typecheck_args(Self(), ClassA)
+			def foo(self, a):
+				return a
+				
+		try:
+			ClassA().foo(ClassA())
+		except TypeCheckError, e:
+			assert isinstance(e.internal, _TC_TypeError)
+			assert e.internal.right == ClassB
+			assert e.internal.wrong == ClassA
+		else:
+			raise AssertionError("Failed to raise TypeCheckError")
 
 ### Bookkeeping ###
 if __name__ == '__main__':
