@@ -1,7 +1,7 @@
 import inspect
 import types
 
-from types import GeneratorType, FunctionType
+from types import GeneratorType, FunctionType, MethodType
 
 # Controls whether typechecking is on (True) or off (False)
 enable_checking = True
@@ -21,7 +21,7 @@ class _TC_Exception(Exception):
 		raise NotImplementedError("Incomplete _TC_Exception subclass (%s)" % str(self.__class__))
 		
 	def format_bad_object(self, bad_object):
-		return ("in %s, " % str(bad_object), self)
+		return ("for %s, " % str(bad_object), self)
 
 class _TC_LengthError(_TC_Exception):
 	def __init__(self, wrong, right=None):
@@ -67,7 +67,7 @@ class _TC_IndexError(_TC_NestedError):
 # message.
 class _TC_DictError(_TC_NestedError):
 	def format_bad_object(self, bad_object):
-		message = "in {%s}, " % ', '.join(["%s: %s" % (repr(k), repr(bad_object[k])) for k in sorted(bad_object.keys())])
+		message = "for {%s}, " % ', '.join(["%s: %s" % (repr(k), repr(bad_object[k])) for k in sorted(bad_object.keys())])
 		
 		if not isinstance(self.inner, _TC_LengthError):
 			return (message, self)
@@ -162,6 +162,25 @@ class _TC_XorError(_TC_NestedError):
 			m = "both assertions"
 	
 		return _TC_NestedError.message(self) + " (matched %s)" % m
+		
+class _TC_FunctionError(_TC_Exception):
+	def __init__(self, checking_func, obj):
+		self.checking_func = checking_func
+		self.rejected_obj = obj
+		
+	def message(self):
+		return " was rejected by %s" % self.checking_func
+		
+	def format_bad_object(self, bad_object):
+		return (str(bad_object), self)
+		
+class _TC_ExactError(_TC_Exception):
+	def __init__(self, wrong, right):
+		self.wrong = wrong
+		self.right = right
+		
+	def message(self):
+		return "expected %s, got %s" % (self.right, self.wrong)
 
 ### The following exist to provide detailed TypeSignatureErrors
 class _TS_Exception(Exception):
@@ -645,9 +664,36 @@ class TypeVariables(CheckType):
 			cls.__active_mapping = cls.__mapping_stack.pop()
 		else:
 			raise TypeError(func)
+			
+class Function(CheckType):
+	def __init__(self, func):
+		self._func = func
+		self.type = self
+	
+	@classmethod
+	def __typesig__(cls, obj):
+		if isinstance(obj, (FunctionType, MethodType)):
+			return cls(obj)
+			
+	def __typecheck__(self, func, to_check):
+		if False == self._func(to_check):
+			raise _TC_FunctionError(self._func, to_check)
+			
+	def __str__(self):
+		return "Function(%s)" % self._func
+		
+	__repr__ = __str__
+	
+	def __eq__(self, other):
+		if self.__class__ is not other.__class__:
+			return False
+		return self._func == other._func
+		
+	def __hash__(self):
+		return hash(str(self.__class__) + str(hash(self._func)))
 		
 # Register some of the above types so that Type() knows about them
-for c in (CheckType, List, Tuple, Dict, Single, TypeVariables):
+for c in (CheckType, List, Tuple, Dict, Single, TypeVariables, Function):
 	register_type(c)
 
 ### The following are utility classes intended to make writing complex
@@ -909,9 +955,100 @@ class YieldSeq(CheckType):
 				
 register_type(YieldSeq)
 
+class Exact(CheckType):
+	def __init__(self, obj):
+		self.type = self
+		self._obj = obj
+		
+	def __hash__(self):
+		try:
+			obj_hash = str(hash(self._obj))
+		except TypeError:
+			obj_hash = str(type(self._obj)) + str(self._obj)
+		
+		return hash(str(self.__class__) + obj_hash)
+		
+	def __eq__(self, other):
+		if self.__class__ is not other.__class__:
+			return False
+		return self._obj == other._obj
+		
+	def __typecheck__(self, func, to_check):
+		if self._obj != to_check:
+			raise _TC_ExactError(to_check, self._obj)
+			
+class Length(CheckType):
+	def __init__(self, length):
+		self.type = self
+		self._length = int(length)
+		
+	def __hash__(self):
+		return hash(str(self.__class__) + str(self._length))
+		
+	def __eq__(self, other):
+		if self.__class__ is not other.__class__:
+			return False
+		return self._length == other._length
+		
+	def __typecheck__(self, func, to_check):
+		try:
+			length = len(to_check)
+		except TypeError:
+			raise _TC_TypeError(to_check, "something with a __len__ method")
+			
+		if length != self._length:
+			raise _TC_LengthError(length, self._length)
+
+import sys			
+class Class(CheckType):
+	def __init__(self, class_name):
+		self.type = self
+		self.class_name = class_name
+		self.class_obj = None
+		self._frame = sys._getframe(1)
+		
+	def __hash__(self):
+		return hash(str(self.__class__) + self.class_name)
+		
+	def __str__(self):
+		return "Class('%s')" % self.class_name
+		
+	__repr__ = __str__
+		
+	def __eq__(self, other):
+		if self.__class__ is not other.__class__:
+			return False
+		return self.class_name == other.class_name
+		
+	def __typecheck__(self, func, to_check):
+		if self.class_obj is None:
+			class_name = self.class_name
+			frame = self._frame
+		
+			for f_dict in (frame.f_locals, frame.f_globals):
+				if class_name in frame.f_locals:
+					if self is not frame.f_locals[class_name]:
+						self.class_obj = frame.f_locals[class_name]
+						self._frame = None
+						break
+			else:
+				raise NameError("name '%s' is not defined" % class_name)
+		
+		if not isinstance(to_check, self.class_obj):
+			raise _TC_TypeError(to_check, self.class_obj)
+
 # The current implementation of Self relies on the TypeVariables machinery
+_Self = TypeVariables("this is the class of the invocant")
 def Self():
-	return 'this is the class of the invocant'
+	return _Self
+	
+### Aliases
+###########
+
+IsOneOf = Or
+IsAllOf = And
+IsNoneOf = Not
+IsOnlyOneOf = Xor
 
 ### This is the public side of the module
 #########################################
@@ -921,14 +1058,15 @@ class TypeCheckException(Exception):
 	pass
 
 class TypeCheckError(TypeCheckException):
-	def __init__(self, bad_object, exception):
-		TypeCheckException.__init__(self, bad_object, exception)
+	def __init__(self, prefix, bad_object, exception):
+		TypeCheckException.__init__(self, prefix, bad_object, exception)
 		
+		self.prefix = prefix
 		self.internal = exception
 		self.bad_object = bad_object
 		
 		(bad_obj_str, start_message) = exception.format_bad_object(bad_object)
-		self.__message = bad_obj_str + start_message.message()
+		self.__message = prefix + bad_obj_str + start_message.message()
 		
 	def __str__(self):
 		return self.__message
@@ -1012,15 +1150,7 @@ def _analyse_params(pos_names, kw_names, vargs, kwargs, handle_missing_kw, tupli
 				continue
 		names[name] = t
 
-	# names is a dictionary at this point, but we'd like
-	# it to be a tuple, s'il vous plait
-	reconstructed = []
-	for name in pos_names:
-		if isinstance(name, list):
-			name = _tuplize(name)
-		reconstructed.append(names.pop(name))
-
-	return tuple(reconstructed), names, vargs, kwargs
+	return names, vargs, kwargs
 
 # This will be invoked when analyse comes upon a keyword argument
 # that it can't find a value for
@@ -1077,6 +1207,12 @@ def _make_fake_function(func):
 	
 	return fake_function
 
+def _tup_to_str(tup):
+	if isinstance(tup, tuple):
+		if len(tup) == 1:
+			return "(" + str(tup[0]) + ",)"
+		return "(" + ", ".join([_tup_to_str(e) for e in tup]) + ")"
+	return str(tup)
 ###################################################
 ### End helper classes/functions for typecheck_args
 
@@ -1084,7 +1220,7 @@ def typecheck_args(*v_sig, **kw_sig):
 	# typecheck_args is run to obtain the real decorator
 	def decorator(func):
 		if hasattr(func, '__wrapped_func'):
-			if hasattr(func, 'type_args_pos'):
+			if hasattr(func, 'type_args'):
 				raise RuntimeError('Cannot use the same typecheck_* function more than once on the same function')
 			wrapped_func = func.__wrapped_func
 		else:
@@ -1104,7 +1240,7 @@ def typecheck_args(*v_sig, **kw_sig):
 		# If anything is amiss, consider the entire signature invalid
 		# and inform the user
 		try:
-			types, kw_types, t_vargs, t_kwargs = _analyse_params(pos_names, kw_names, v_sig, kw_sig, _types__missing_kw, _tuplize_and_check)
+			types, t_vargs, t_kwargs = _analyse_params(pos_names, kw_names, v_sig, kw_sig, _types__missing_kw, _tuplize_and_check)
 		except _TS_Exception, e:
 			raise TypeSignatureError(e)
 		
@@ -1132,47 +1268,39 @@ def typecheck_args(*v_sig, **kw_sig):
 		# in use, but there are leftover positional or keyword arguments,
 		# consider the signature invalid
 		if kwarg_name is not None:
-			t_kwargs = fixed_dict.pop(kwarg_name)
+			types[kwarg_name] = fixed_dict.pop(kwarg_name)
 		elif len(t_kwargs) > 0:
 			for k in t_kwargs:
-				if k in kw_types:
+				if k in types:
 					param = t_kwargs.keys()[0]
-					raise TypeSignatureError(_TS_TwiceTypedError(param, t_kwargs[param], kw_types[param]))
+					raise TypeSignatureError(_TS_TwiceTypedError(param, t_kwargs[param], types[param]))
 			raise TypeSignatureError(_TS_ExtraKeywordError(t_kwargs.keys()[0]))
 		if varg_name is not None:
-			t_vargs = fixed_dict.pop(varg_name)
+			types[varg_name] = fixed_dict.pop(varg_name)
 		elif len(t_vargs) > 0:
 			raise TypeSignatureError(_TS_ExtraPositionalError(t_vargs[0]))
 		#####################################################################
 		### /Fix-up
 				
 		# Convert the signatures to types now, rather than rebuild them in every function call
-		T_kw_types = dict([(k, Type(v)) for (k, v) in kw_types.items()])
-		T_types = Type(types)
-		T_t_vargs = Type(t_vargs)
-		T_t_kwargs = Type(t_kwargs)
+		T_types = dict([(k, Type(v)) for (k, v) in types.items()])
 
 		def __check_args(*__vargs, **__kwargs):
 			# Type-checking can be turned on and off by toggling the
 			# value of the global enable_checking variable
 			if enable_checking:
-				args, kw_args, a_vargs, a_kwargs = _analyse_params(pos_names, kw_names, __vargs, __kwargs, _args__handle_missing, _tuplize)
+				args, a_vargs, a_kwargs = _analyse_params(pos_names, kw_names, __vargs, __kwargs, _args__handle_missing, _tuplize)
 
 				# Type-check the keyword arguments
-				for (name, val) in kw_args.items():
-					type = T_kw_types[name]
-					try:
-						check_type(type, wrapped_func, val)
-					except _TC_Exception, e:
-						middle_exc = _TC_KeyValError(name, val, e)
-						raise TypeCheckError(kw_args, middle_exc)
-
-				# Type-check everything else
-				for (arg, type) in ((args, T_types), (a_vargs, T_t_vargs), (a_kwargs, T_t_kwargs)):
-					try:
-						check_type(type, wrapped_func, arg)
-					except _TC_Exception, e:
-						raise TypeCheckError(arg, e)
+				try:
+					for name, val in args.items():
+						check_type(T_types[name], wrapped_func, val)
+					
+					for name, val in ((varg_name, a_vargs), (kwarg_name, a_kwargs)):
+						if name:
+							check_type(T_types[name], wrapped_func, val)
+				except _TC_Exception, e:
+					raise TypeCheckError("Argument %s: " % _tup_to_str(name), val, e)
 		
 		if hasattr(func, '__check_result'):
 			# This is one of our wrapper functions, probably created by
@@ -1186,20 +1314,11 @@ def typecheck_args(*v_sig, **kw_sig):
 		fake_function.__check_args = __check_args
 		
 		### Add the publically-accessible signature information
-		fake_function.type_args_pos = types
-		fake_function.type_args_kw = kw_types
-		
-		# Adjust these so that they're consistent
-		for (attr, var) in (('vargs', t_vargs), ('kwargs', t_kwargs)):
-			if isinstance(var, Empty):
-				var = var.type()
-			setattr(fake_function, 'type_args_%s' % attr, var)
+		fake_function.type_args = types
+
 		return fake_function
 	return decorator
 	
-# This is for backwards compatibility with v0.1.6 and earlier
-typecheck = typecheck_args
-
 # Refactor this out of typecheck_{return,yield}	
 def _decorator(signature, conflict_field, twice_field, check_result_func):
 	def decorator(func):
@@ -1231,7 +1350,7 @@ def typecheck_return(*signature):
 				check_type(sig_types, func, return_vals)
 			except _TC_Exception, e:
 				stop_checking(func)
-				raise TypeCheckError(return_vals, e)
+				raise TypeCheckError("Return value: ", return_vals, e)
 
 		stop_checking(func)
 		return return_vals
@@ -1270,7 +1389,7 @@ class Fake_generator(object):
 				# Insert this error into the chain so we can know
 				# which yield the error occurred at
 				middle_exc = _TC_GeneratorError(self.__yield_no, e)
-				raise TypeCheckError(return_vals, middle_exc)
+				raise TypeCheckError("", return_vals, middle_exc)
 
 		# Everything checks out. Return the results
 		return return_vals
@@ -1299,3 +1418,9 @@ def typecheck_yield(*signature):
 		# Otherwise, we build ourselves a fake generator
 		return Fake_generator(gen, signature)
 	return _decorator(signature, 'type_return', 'type_yield', __check_yield)
+
+# Aliases
+typecheck = typecheck_args
+accepts = typecheck_args
+returns = typecheck_return
+yields = typecheck_yield
