@@ -1,5 +1,6 @@
 from typecheck import CheckType, _TC_TypeError, check_type, Type
 from typecheck import register_type, Or, _TC_Exception, _TC_KeyError
+from typecheck import _TC_LengthError
 
 ### Provide typechecking for the built-in set() class
 ###
@@ -10,9 +11,11 @@ class Set(CheckType):
 		self.type = set(set_list)
 		self._types = [Type(t) for t in self.type]
 		
+		# self._type is used to build _TC_TypeError
 		if len(self._types) > 1:
-			self._type = Or(*[t for t in self.type])
-		else:
+			self._type = Or(*self.type)
+		elif len(self._types) == 1:
+			# XXX Is there an easier way to get this?
 			t = self.type.pop()
 			self._type = t
 			self.type.add(t)
@@ -26,6 +29,9 @@ class Set(CheckType):
 		if not isinstance(to_check, set):
 			raise _TC_TypeError(to_check, self.type)
 			
+		if len(self._types) == 0 and len(to_check) > 0:
+			raise _TC_LengthError(len(to_check), 0)
+			
 		for obj in to_check:
 			error = False
 			for type in self._types:
@@ -43,7 +49,10 @@ class Set(CheckType):
 	def __eq__(self, other):
 		if self.__class__ is not other.__class__:
 			return False
-		return self.type.issubset(other.type)
+		return self.type == other.type
+		
+	def __hash__(self):
+		return hash(str(hash(self.__class__)) + str(hash(frozenset(self.type))))
 			
 	@classmethod
 	def __typesig__(self, obj):
